/* $Id: socklib.c,v 1.1 1998/12/28 15:26:42 mas Rel $
 * Copyright 1998 Marc Andr Selig.
 * All rights reserved.
 * Socket library.
 */

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <netdb.h>



void init_sockaddr (struct sockaddr_in *name,
                    const char *hostname,
                    unsigned short int port)
{	/* originally from glibc2 info */
  struct hostent *hostinfo;

  name->sin_family = AF_INET;
  name->sin_port = htons (port);
  hostinfo = gethostbyname (hostname);
  if (hostinfo == NULL) {
    fprintf (stderr, "Unknown host %s.\n", hostname);
    exit (EXIT_FAILURE);
  }
  name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
}



int make_socket (unsigned short int port)
     /* returns the fd to the socket */
{
  int sock;
  struct sockaddr_in name;

  /* Create the socket. */
  sock = socket (PF_INET, SOCK_STREAM, 0);
  if (sock < 0) {
      perror ("socket");
      exit (EXIT_FAILURE);
  }

  /* Give the socket a name. */
  name.sin_family = AF_INET;
  name.sin_port = htons (port);
  name.sin_addr.s_addr = htonl (INADDR_ANY);
  if (bind (sock, (struct sockaddr *) &name, sizeof (name)) < 0) {
    perror ("bind");
    exit (EXIT_FAILURE);
  }

  return sock;
}



int read_from_socket (int filedes, char *message, int max_msg_len)
     /* will return -2 on read error, -1 on EOF, 0 on success */
{
  int nbytes;

  nbytes = read (filedes, message, max_msg_len);
  if (nbytes < 0) {
    /* Read error. */
    return -2;
  } else if (nbytes == 0)
    /* End-of-file. */
    return -1;
  else
    /* Data read. */
    return 0;
}



int write_to_socket (int filedes, char *message)
     /* will return -1 on error, 0 on success */
{
  int nbytes;

  nbytes = write (filedes, message, strlen (message) + 1);
  if (nbytes < 0) {
    return -1;
  } else
    return 0;
}
